@@ -5,19 +5,23 @@ use std::ptr;
 use std::sync::Mutex;
 
 use super::{
-    AtcaDeviceType, AtcaIfaceCfg, AtcaIfaceCfgPtrWrapper, AtcaIfaceType, AtcaSlot,
-    AtcaSlotCapacity, AtcaStatus, AteccDeviceTrait, ChipOptions, EccKeyAttr, InfoCmdType, KeyType,
-    NonceTarget, OutputProtectionState, ReadKey, SignMode, SlotConfig, VerifyMode, WriteConfig,
+    AeadAlgorithm, AeadParam, AtcaAesCcmCtx, AtcaDeviceType, AtcaIfaceCfg, AtcaIfaceCfgPtrWrapper,
+    AtcaIfaceType, AtcaSlot, AtcaSlotCapacity, AtcaStatus, AteccDeviceTrait, ChipOptions,
+    EccKeyAttr, InfoCmdType, KeyType, NonceTarget, OutputProtectionState, ReadKey, SignMode,
+    SlotConfig, VerifyMode, WriteConfig,
 };
 use super::{
-    ATCA_AES_KEY_SIZE, ATCA_ATECC_CONFIG_BUFFER_SIZE, ATCA_ATECC_MIN_SLOT_IDX_FOR_PUB_KEY,
-    ATCA_ATECC_PRIV_KEY_SIZE, ATCA_ATECC_PUB_KEY_SIZE, ATCA_ATECC_SLOTS_COUNT,
-    ATCA_ATECC_TEMPKEY_KEYID, ATCA_ATSHA_CONFIG_BUFFER_SIZE, ATCA_BLOCK_SIZE, ATCA_KEY_SIZE,
-    ATCA_LOCK_ZONE_CONFIG, ATCA_LOCK_ZONE_DATA, ATCA_NONCE_NUMIN_SIZE, ATCA_NONCE_SIZE,
-    ATCA_RANDOM_BUFFER_SIZE, ATCA_SERIAL_NUM_SIZE, ATCA_SHA2_256_DIGEST_SIZE, ATCA_SIG_SIZE,
-    ATCA_ZONE_CONFIG, ATCA_ZONE_DATA,
+    ATCA_AES_DATA_SIZE, ATCA_AES_GCM_IV_STD_LENGTH, ATCA_AES_KEY_SIZE,
+    ATCA_ATECC_CONFIG_BUFFER_SIZE, ATCA_ATECC_MIN_SLOT_IDX_FOR_PUB_KEY, ATCA_ATECC_PRIV_KEY_SIZE,
+    ATCA_ATECC_PUB_KEY_SIZE, ATCA_ATECC_SLOTS_COUNT, ATCA_ATECC_TEMPKEY_KEYID,
+    ATCA_ATSHA_CONFIG_BUFFER_SIZE, ATCA_BLOCK_SIZE, ATCA_KEY_SIZE, ATCA_LOCK_ZONE_CONFIG,
+    ATCA_LOCK_ZONE_DATA, ATCA_NONCE_NUMIN_SIZE, ATCA_NONCE_SIZE, ATCA_RANDOM_BUFFER_SIZE,
+    ATCA_SERIAL_NUM_SIZE, ATCA_SHA2_256_DIGEST_SIZE, ATCA_SIG_SIZE, ATCA_ZONE_CONFIG,
+    ATCA_ZONE_DATA,
 };
 
+mod aes_ccm;
+mod aes_gcm;
 mod c2rust;
 mod rust2c;
 
@@ -158,6 +162,28 @@ impl AteccDeviceTrait for AteccDevice {
         self.verify_hash(mode, hash, signature)
     } // AteccDevice::verify_hash()
 
+    /// Data encryption function in AES AEAD (authenticated encryption with associated data) modes
+    /// Trait implementation
+    fn aead_encrypt(
+        &self,
+        algorithm: AeadAlgorithm,
+        slot_id: u8,
+        data: &mut [u8],
+    ) -> Result<Vec<u8>, AtcaStatus> {
+        self.aead_encrypt(algorithm, slot_id, data)
+    } // AteccDevice::aead_encrypt()
+
+    /// Data decryption function in AES AEAD (authenticated encryption with associated data) modes
+    /// Trait implementation
+    fn aead_decrypt(
+        &self,
+        algorithm: AeadAlgorithm,
+        slot_id: u8,
+        data: &mut [u8],
+    ) -> Result<bool, AtcaStatus> {
+        self.aead_decrypt(algorithm, slot_id, data)
+    } // AteccDevice::aead_decrypt()
+
     /// Request ATECC to return own device type
     /// Trait implementation
     fn get_device_type(&self) -> AtcaDeviceType {
@@ -197,7 +223,7 @@ impl AteccDeviceTrait for AteccDevice {
     /// Trait implementation
     fn add_access_key(&self, slot_id: u8, access_key: &[u8]) -> AtcaStatus {
         self.add_access_key(slot_id, access_key)
-    } // AteccDevice::add_access_key
+    } // AteccDevice::add_access_key()
 
     /// A function that deletes all access keys for secure read or write operations
     /// performed by the ATECCx08 chip
@@ -296,7 +322,7 @@ impl AteccDeviceTrait for AteccDevice {
     #[cfg(test)]
     fn get_access_key(&self, slot_id: u8, key: &mut Vec<u8>) -> AtcaStatus {
         self.get_access_key(slot_id, key)
-    }
+    } // AteccDevice::get_access_key()
 }
 
 /// Implementation of CryptoAuth Library API Rust wrapper calls
@@ -462,12 +488,23 @@ impl AteccDevice {
     /// Digest Buffer (32 or 64 bytes), or the Alternate Key Buffer (32 bytes). For
     /// all other devices, only TempKey (32 bytes) is available.
     fn nonce(&self, target: NonceTarget, data: &[u8]) -> AtcaStatus {
-        if (self.get_device_type() != AtcaDeviceType::ATECC608A)
-            && (target != NonceTarget::TempKey)
-            && (data.len() != ATCA_NONCE_SIZE)
+        if (self.get_device_type() != AtcaDeviceType::ATECC608A) && (target != NonceTarget::TempKey)
         {
             return AtcaStatus::AtcaBadParam;
         }
+        let dev_type_608: bool = AtcaDeviceType::ATECC608A == self.get_device_type();
+        let alt_key_buff: bool = NonceTarget::AltKeyBuf == target;
+        let no_len_32: bool = data.len() != ATCA_NONCE_SIZE;
+        let no_len_64: bool = data.len() != (2 * ATCA_NONCE_SIZE);
+
+        if !dev_type_608 && alt_key_buff
+            || alt_key_buff && no_len_32
+            || !dev_type_608 && !no_len_64
+            || no_len_32 && no_len_64
+            || !no_len_32 && !no_len_64
+        {
+            return AtcaStatus::AtcaInvalidSize;
+        }
         AtcaStatus::from(unsafe {
             let _guard = self
                 .api_mutex
@@ -687,6 +724,9 @@ impl AteccDevice {
     /// public key based on an existing private key in the socket
     /// or exports the public key directly
     fn get_public_key(&self, slot_id: u8, public_key: &mut Vec<u8>) -> AtcaStatus {
+        if self.check_that_configuration_is_not_locked(true) {
+            return AtcaStatus::AtcaNotLocked;
+        }
         if self.slots[slot_id as usize].config.key_type != KeyType::P256EccKey {
             return AtcaStatus::AtcaBadParam;
         }
@@ -826,6 +866,48 @@ impl AteccDevice {
         }
     } // AteccDevice::verify_hash()
 
+    /// Data encryption function in AES AEAD (authenticated encryption with associated data) modes
+    fn aead_encrypt(
+        &self,
+        algorithm: AeadAlgorithm,
+        slot_id: u8,
+        data: &mut [u8],
+    ) -> Result<Vec<u8>, AtcaStatus> {
+        if self.check_that_configuration_is_not_locked(true) {
+            return Err(AtcaStatus::AtcaNotLocked);
+        }
+        if !self.is_aes_enabled() {
+            // If chip does not support AES hardware encryption, the operation cannot be performed
+            return Err(AtcaStatus::AtcaBadParam);
+        }
+
+        match algorithm {
+            AeadAlgorithm::Ccm(aead_param) => self.encrypt_aes_ccm(aead_param, slot_id, data),
+            AeadAlgorithm::Gcm(aead_param) => self.encrypt_aes_gcm(aead_param, slot_id, data),
+        }
+    } // AteccDevice::aead_encrypt()
+
+    /// Data decryption function in AES AEAD (authenticated encryption with associated data) modes
+    fn aead_decrypt(
+        &self,
+        algorithm: AeadAlgorithm,
+        slot_id: u8,
+        data: &mut [u8],
+    ) -> Result<bool, AtcaStatus> {
+        if self.check_that_configuration_is_not_locked(true) {
+            return Err(AtcaStatus::AtcaNotLocked);
+        }
+        if !self.is_aes_enabled() {
+            // If chip does not support AES hardware encryption, the operation cannot be performed
+            return Err(AtcaStatus::AtcaBadParam);
+        }
+
+        match algorithm {
+            AeadAlgorithm::Ccm(aead_param) => self.decrypt_aes_ccm(aead_param, slot_id, data),
+            AeadAlgorithm::Gcm(aead_param) => self.decrypt_aes_gcm(aead_param, slot_id, data),
+        }
+    } // AteccDevice::aead_decrypt()
+
     /// Request ATECC to return own device type
     fn get_device_type(&self) -> AtcaDeviceType {
         AtcaDeviceType::from(unsafe {
