@@ -1,9 +1,10 @@
 use super::{
-    AtcaDeviceType, AtcaIfaceCfg, AtcaIfaceType, AtcaSlot, AtcaStatus, AteccDeviceTrait,
-    InfoCmdType, KeyType, NonceTarget, OutputProtectionState, SignMode, VerifyMode,
+    AeadAlgorithm, AtcaDeviceType, AtcaIfaceCfg, AtcaIfaceType, AtcaSlot, AtcaStatus,
+    AteccDeviceTrait, InfoCmdType, KeyType, NonceTarget, OutputProtectionState, SignMode,
+    VerifyMode,
 };
 
-use super::{ATCA_RANDOM_BUFFER_SIZE, ATCA_SERIAL_NUM_SIZE};
+use super::{ATCA_AES_DATA_SIZE, ATCA_RANDOM_BUFFER_SIZE, ATCA_SERIAL_NUM_SIZE};
 use rand::{distributions::Standard, Rng};
 
 pub struct AteccDevice {
@@ -91,6 +92,30 @@ impl AteccDeviceTrait for AteccDevice {
             _ => Err(self.default_dev_status()),
         }
     }
+    /// Data encryption function in AES AEAD (authenticated encryption with associated data) modes
+    fn aead_encrypt(
+        &self,
+        _algorithm: AeadAlgorithm,
+        _slot_id: u8,
+        _data: &mut [u8],
+    ) -> Result<Vec<u8>, AtcaStatus> {
+        match self.dev_type {
+            AtcaDeviceType::AtcaTestDevSuccess => Ok(vec![0; ATCA_AES_DATA_SIZE]),
+            _ => Err(self.default_dev_status()),
+        }
+    }
+    /// Data decryption function in AES AEAD (authenticated encryption with associated data) modes
+    fn aead_decrypt(
+        &self,
+        _algorithm: AeadAlgorithm,
+        _slot_id: u8,
+        _data: &mut [u8],
+    ) -> Result<bool, AtcaStatus> {
+        match self.dev_type {
+            AtcaDeviceType::AtcaTestDevSuccess => Ok(true),
+            _ => Err(self.default_dev_status()),
+        }
+    }
     /// Request ATECC to return own device type
     fn get_device_type(&self) -> AtcaDeviceType {
         self.dev_type
