@@ -1,3 +1,8 @@
+use std::mem::MaybeUninit;
+
+use cryptoauthlib_sys::atca_aes_ctr_ctx_t;
+use cryptoauthlib_sys::atca_aes_cmac_ctx_t;
+
 /// An ATECC/ATSHA device buffer to load
 #[repr(u8)]
 #[derive(Copy, Clone, Debug, PartialEq)]
@@ -76,6 +81,80 @@ impl Default for VerifyEcdsaParam {
     }
 }
 
+/// Type of AEAD algorithm
+#[derive(Clone, Debug, PartialEq)]
+pub enum AeadAlgorithm {
+    Ccm(AeadParam),
+    Gcm(AeadParam),
+}
+
+/// AEAD algorithm parameters for compute
+#[derive(Clone, Debug, PartialEq)]
+pub struct AeadParam {
+    /// Nonce [number used once aka IV] (default length is 12 bytes)
+    pub nonce: Vec<u8>,
+    /// external encryption/decryption key needed when an AES key stored in the cryptochip is not used
+    pub key: Option<[u8; ATCA_AES_KEY_SIZE]>,
+    /// tag to verify authenticity of decrypted data (16 bytes)
+    pub tag: Option<Vec<u8>>,
+    /// tag length generated during encryption
+    pub tag_length: Option<u8>,
+    /// Additional data that will be authenticated but not encrypted
+    pub additional_data: Option<Vec<u8>>,
+}
+
+impl Default for AeadParam {
+    fn default() -> AeadParam {
+        AeadParam {
+            nonce: Vec::new(),
+            key: None,
+            tag: None,
+            tag_length: None,
+            additional_data: None,
+        }
+    }
+}
+
+/// Data context structure for AEAD encryption in CCM mode
+#[derive(Copy, Clone, Debug)]//, PartialEq)]
+pub struct AtcaAesCcmCtx {
+    pub cbc_mac_ctx: atca_aes_cmac_ctx_t,           // CBC_MAC context
+    pub ctr_ctx: atca_aes_ctr_ctx_t,                // CTR context
+    pub iv_size: u8,                                // iv size
+    pub m: u8,                                      // Tag size
+    pub counter: [u8; ATCA_AES_DATA_SIZE],          // Initial counter value
+    pub partial_aad: [u8; ATCA_AES_DATA_SIZE],      // Partial blocks of data waiting to be processed
+    pub partial_aad_size: usize,                    // Amount of data in the partial block buffer
+    pub text_size: usize,                           // Size of data to be processed
+    pub enc_cb: [u8; ATCA_AES_DATA_SIZE],           // Last encrypted counter block
+    pub data_size: u32,                             // Size of the data being encrypted/decrypted in bytes.
+    pub ciphertext_block: [u8; ATCA_AES_DATA_SIZE]  // Last ciphertext block
+}
+
+impl Default for AtcaAesCcmCtx {
+    fn default() -> AtcaAesCcmCtx {
+        AtcaAesCcmCtx {
+            cbc_mac_ctx: {
+                let ctx = MaybeUninit::<atca_aes_cmac_ctx_t>::zeroed();
+                unsafe { ctx.assume_init() }
+            },
+            ctr_ctx: {
+                let ctx = MaybeUninit::<atca_aes_ctr_ctx_t>::zeroed();
+                unsafe { ctx.assume_init() }
+            },
+            iv_size: ATCA_AES_DATA_SIZE as u8,
+            m: ATCA_AES_DATA_SIZE as u8,
+            counter: [0x00; ATCA_AES_DATA_SIZE],
+            partial_aad: [0x00; ATCA_AES_DATA_SIZE],
+            partial_aad_size: 0,
+            text_size: 0,
+            enc_cb: [0x00; ATCA_AES_DATA_SIZE],
+            data_size: 0,
+            ciphertext_block: [0x00; ATCA_AES_DATA_SIZE],
+        }
+    }
+}
+
 /// structure that stores data for options supported by the chip
 #[derive(Copy, Clone, Debug, PartialEq)]
 pub struct ChipOptions {
@@ -110,7 +189,8 @@ impl Default for ChipOptions {
     }
 }
 
-/// 
+/// Allowed IO transmission states between chip and host MCU
+/// for ECDH, KDF, Verify and SecureBoot commands.
 #[repr(u8)]
 #[derive(Copy, Clone, Debug, PartialEq)]
 pub enum OutputProtectionState {
