@@ -1,113 +1,23 @@
-use serde::Deserialize;
-use serial_test::serial;
-use std::fs::read_to_string;
-use std::path::Path;
-
 // Types
 use super::{
-    AtcaIface, AtcaIfaceCfg, AtcaIfaceI2c, AtcaSlot, AtcaStatus, AteccDevice, InfoCmdType, KeyType,
-    NonceTarget, SignEcdsaParam, SignMode, VerifyEcdsaParam, VerifyMode,
+    AtcaSlot, AtcaStatus, InfoCmdType, KeyType, NonceTarget, SignEcdsaParam, SignMode,
+    VerifyEcdsaParam, VerifyMode,
 };
 // Constants
 use super::{
     ATCA_AES_KEY_SIZE, ATCA_ATECC_PUB_KEY_SIZE, ATCA_ATECC_SLOTS_COUNT, ATCA_NONCE_NUMIN_SIZE,
-    ATCA_RANDOM_BUFFER_SIZE, ATCA_SIG_SIZE, ATCA_ZONE_CONFIG,
+    ATCA_RANDOM_BUFFER_SIZE, ATCA_SIG_SIZE,
 };
-// Functions
+
+use super::hw_backend_common::*;
 use super::hw_impl::atcab_get_config_from_config_zone;
-use super::setup_atecc_device;
-
-#[derive(Deserialize)]
-struct Config {
-    pub device: Device,
-    pub interface: Option<Interface>,
-}
-
-#[derive(Deserialize)]
-struct Device {
-    pub device_type: String,
-    pub iface_type: String,
-    pub wake_delay: Option<u16>,
-    pub rx_retries: Option<i32>,
-}
-
-#[derive(Deserialize, Copy, Clone)]
-struct Interface {
-    pub slave_address: u8,
-    pub bus: u8,
-    pub baud: u32,
-}
-
-fn is_chip_version_608(device: &AteccDevice) -> Result<bool, AtcaStatus> {
-    const LEN: u8 = 4;
-    const OFFSET_REV: u8 = 1;
-    const INDEX_OF_REV: usize = 2;
-
-    let mut data: Vec<u8> = Vec::with_capacity(LEN as usize);
-
-    let result_dev_type = device.read_zone(ATCA_ZONE_CONFIG, 0, 0, OFFSET_REV, &mut data, LEN);
-
-    match result_dev_type {
-        AtcaStatus::AtcaSuccess => Ok((data[INDEX_OF_REV] & 0xF0) == 0x60),
-        _ => Err(result_dev_type),
-    }
-}
-
-fn iface_setup(config_file: String) -> Result<AtcaIfaceCfg, String> {
-    let config_path = Path::new(&config_file);
-    let config_string = read_to_string(config_path).expect("file not found");
-    let config: Config = toml::from_str(&config_string).unwrap();
-    let iface_cfg = AtcaIfaceCfg::default();
-
-    match config.device.iface_type.as_str() {
-        "i2c" => Ok(iface_cfg
-            .set_iface_type("i2c".to_owned())
-            .set_devtype(config.device.device_type)
-            .set_wake_delay(config.device.wake_delay.unwrap())
-            .set_rx_retries(config.device.rx_retries.unwrap())
-            .set_iface(
-                AtcaIface::default().set_atcai2c(
-                    AtcaIfaceI2c::default()
-                        .set_slave_address(config.interface.unwrap().slave_address)
-                        .set_bus(config.interface.unwrap().bus)
-                        .set_baud(config.interface.unwrap().baud),
-                ),
-            )),
-        "test-interface" => Ok(iface_cfg
-            .set_iface_type("test-interface".to_owned())
-            .set_devtype(config.device.device_type.as_str().to_owned())),
-        _ => Err("unsupported interface type".to_owned()),
-    }
-}
-
-/// Setup tests.
-///
-/// # Arguments
-/// * 'data_zone_must_be_locked == true' prevents further calls if data zone is not locked
-/// * 'data_zone_must_be_locked == false' allows further calls even if data zone is not locked
-pub fn test_setup() -> AteccDevice {
-    let result_iface_cfg = iface_setup("config.toml".to_owned());
-    assert_eq!(result_iface_cfg.is_ok(), true);
-
-    let iface_cfg = result_iface_cfg.unwrap();
-    assert_eq!(iface_cfg.iface_type.to_string(), "AtcaI2cIface");
-
-    let result = setup_atecc_device(iface_cfg);
-    match result {
-        Ok(_) => (),
-        Err(err) => panic!("{}", err),
-    };
-
-    result.unwrap()
-}
-
-// test_teardown() is not needed, it is a one-liner and if it fails, then
-// there is a larger problem - elsewhere...
+use serial_test::serial;
 
 #[test]
 #[serial]
 fn new() {
     const SLOTS_COUNT: usize = ATCA_ATECC_SLOTS_COUNT as usize;
+
     let device = test_setup();
 
     let serial_number = device.get_serial_number();
@@ -143,14 +53,16 @@ fn sha() {
 
     let mut expected = AtcaStatus::AtcaSuccess;
     if !device.is_configuration_locked() {
-        print!("\u{001b}[1m\u{001b}[33mConfiguration not Locked!\u{001b}[0m ");
+        println!("\u{001b}[1m\u{001b}[33mConfiguration not Locked!\u{001b}[0m ");
         expected = AtcaStatus::AtcaNotLocked;
-    } else {
-        assert_eq!(digest, test_message_hash);
-    };
+    }
+
     assert_eq!(device.release().to_string(), "AtcaSuccess");
 
     assert_eq!(device_sha, expected);
+    if AtcaStatus::AtcaSuccess == expected {
+        assert_eq!(digest, test_message_hash);
+    }
 }
 
 #[test]
@@ -184,10 +96,10 @@ fn nonce() {
 
     assert_eq!(device.release().to_string(), "AtcaSuccess");
 
-    assert_eq!(nonce_32_ok.to_string(), "AtcaSuccess");
+    assert_eq!(nonce_32_ok, AtcaStatus::AtcaSuccess);
     assert_eq!(nonce_64_ok, expected);
-    assert_eq!(nonce_bad.to_string(), "AtcaBadParam");
-    assert_eq!(check_ver_result.to_string(), "AtcaSuccess");
+    assert_eq!(nonce_bad, AtcaStatus::AtcaInvalidSize);
+    assert_eq!(check_ver_result, AtcaStatus::AtcaSuccess);
 }
 
 #[test]
@@ -208,8 +120,8 @@ fn nonce_rand() {
     assert_eq!(device.release().to_string(), "AtcaSuccess");
 
     assert_eq!(rand_out.len(), ATCA_RANDOM_BUFFER_SIZE);
-    assert_eq!(nonce_ok.to_string(), "AtcaSuccess");
-    assert_eq!(nonce_bad.to_string(), "AtcaInvalidSize");
+    assert_eq!(nonce_ok, AtcaStatus::AtcaSuccess);
+    assert_eq!(nonce_bad, AtcaStatus::AtcaInvalidSize);
 }
 
 #[test]
@@ -225,68 +137,64 @@ fn gen_key() {
         0x66, 0x45,
     ];
 
-    let mut check_ver_result: AtcaStatus = AtcaStatus::AtcaSuccess;
-    let mut expected_device_gen_key_bad_2 = match is_chip_version_608(&device) {
-        Ok(true) => AtcaStatus::AtcaSuccess,
-        Ok(false) => AtcaStatus::AtcaBadParam,
-        Err(err) => {
-            check_ver_result = err;
-            AtcaStatus::AtcaBadParam
-        }
-    };
-
+    let mut chip_is_locked: bool = true;
+    let mut expected_device_gen_key_ok_1 = AtcaStatus::AtcaSuccess;
+    let mut expected_device_gen_key_ok_2 = AtcaStatus::AtcaSuccess;
+    let mut expected_device_gen_key_ok_3 = AtcaStatus::AtcaSuccess;
     let mut expected_device_gen_key_bad_1 = AtcaStatus::AtcaInvalidId;
+    let mut expected_device_gen_key_bad_2 = AtcaStatus::AtcaBadParam;
     let mut expected_device_gen_key_bad_3 = AtcaStatus::AtcaBadParam;
     let mut expected_device_gen_key_bad_4 = AtcaStatus::AtcaBadParam;
-    let mut expected_device_gen_key_ok_1 = AtcaStatus::AtcaSuccess;
+
     if !device.is_configuration_locked() {
-        print!("\u{001b}[1m\u{001b}[33mConfiguration not Locked!\u{001b}[0m ");
+        println!("\u{001b}[1m\u{001b}[33mConfiguration not Locked!\u{001b}[0m ");
+        chip_is_locked = false;
+
+        expected_device_gen_key_ok_1 = AtcaStatus::AtcaNotLocked;
+        expected_device_gen_key_ok_2 = AtcaStatus::AtcaNotLocked;
+        expected_device_gen_key_ok_3 = AtcaStatus::AtcaNotLocked;
         expected_device_gen_key_bad_1 = AtcaStatus::AtcaNotLocked;
         expected_device_gen_key_bad_2 = AtcaStatus::AtcaNotLocked;
         expected_device_gen_key_bad_3 = AtcaStatus::AtcaNotLocked;
         expected_device_gen_key_bad_4 = AtcaStatus::AtcaNotLocked;
-        expected_device_gen_key_ok_1 = AtcaStatus::AtcaNotLocked;
     }
-    let device_gen_key_bad_1 = device.gen_key(KeyType::Aes, ATCA_ATECC_SLOTS_COUNT + 1);
-    let device_gen_key_bad_2 = device.gen_key(KeyType::Aes, 9);
-    let device_gen_key_bad_3 = device.gen_key(KeyType::P256EccKey, ATCA_ATECC_SLOTS_COUNT);
-    let device_gen_key_bad_4 = device.gen_key(KeyType::ShaOrText, 0);
-    let device_gen_key_ok_1 = device.gen_key(KeyType::P256EccKey, 0);
-
-    match is_chip_version_608(&device) {
-        Ok(true) => {
-            let aes_key_ok_1 = device.gen_key(KeyType::Aes, 0x09);
-            assert_eq!(aes_key_ok_1.to_string(), "AtcaSuccess");
-
-            let write_key_set_success = device.add_access_key(ENCRYPTION_KEY_SLOT, &write_key);
-            assert_eq!(write_key_set_success.to_string(), "AtcaSuccess");
-
-            if AtcaStatus::AtcaSuccess == write_key_set_success {
-                let aes_key_ok_2 = device.gen_key(KeyType::Aes, 0x04);
-                // let aes_key_bad_1 = device.gen_key(KeyType::Aes, &aes_key_bad, 0x09);
-
-                assert_eq!(aes_key_ok_2.to_string(), "AtcaSuccess");
-                // assert_eq!(aes_key_bad_1.to_string(), "AtcaInvalidSize");
-            }
+    if chip_is_locked {
+        if device.is_aes_enabled() {
+            expected_device_gen_key_ok_2 = AtcaStatus::AtcaSuccess;
+            expected_device_gen_key_ok_3 = AtcaStatus::AtcaSuccess;
+            expected_device_gen_key_bad_2 = AtcaStatus::AtcaBadParam;
+        } else {
+            expected_device_gen_key_ok_2 = AtcaStatus::AtcaBadParam;
+            expected_device_gen_key_ok_3 = AtcaStatus::AtcaBadParam;
         }
-        Ok(false) => (),
-        Err(err) => panic!("is_chip_version_608() error: {}", err.to_string()),
     }
 
+    let write_key_set_success = device.add_access_key(ENCRYPTION_KEY_SLOT, &write_key);
+    let device_gen_key_ok_1 = device.gen_key(KeyType::P256EccKey, 0x00);
+    let device_gen_key_ok_2 = device.gen_key(KeyType::Aes, 0x09);
+    let device_gen_key_ok_3 = device.gen_key(KeyType::Aes, 0x04);
+    let device_gen_key_bad_1 = device.gen_key(KeyType::Aes, ATCA_ATECC_SLOTS_COUNT + 1);
+    let device_gen_key_bad_2 = device.gen_key(KeyType::Aes, 0x00);
+    let device_gen_key_bad_3 = device.gen_key(KeyType::P256EccKey, ATCA_ATECC_SLOTS_COUNT);
+    let device_gen_key_bad_4 = device.gen_key(KeyType::ShaOrText, 0x00);
+
     assert_eq!(device.release().to_string(), "AtcaSuccess");
 
-    assert_eq!(check_ver_result.to_string(), "AtcaSuccess");
+    assert_eq!(write_key_set_success, AtcaStatus::AtcaSuccess);
+    assert_eq!(device_gen_key_ok_1, expected_device_gen_key_ok_1);
+    assert_eq!(device_gen_key_ok_2, expected_device_gen_key_ok_2);
+    assert_eq!(device_gen_key_ok_3, expected_device_gen_key_ok_3);
     assert_eq!(device_gen_key_bad_1, expected_device_gen_key_bad_1);
     assert_eq!(device_gen_key_bad_2, expected_device_gen_key_bad_2);
     assert_eq!(device_gen_key_bad_3, expected_device_gen_key_bad_3);
     assert_eq!(device_gen_key_bad_4, expected_device_gen_key_bad_4);
-    assert_eq!(device_gen_key_ok_1, expected_device_gen_key_ok_1);
 }
 
 #[test]
 #[serial]
 fn import_key() {
     const ENCRYPTION_KEY_SLOT: u8 = 0x06;
+
     let device = test_setup();
 
     let priv_key = [
@@ -311,63 +219,73 @@ fn import_key() {
     let aes_key = &priv_key[0..=15];
     let aes_key_bad = &priv_key[0..=10];
 
+    let mut chip_is_locked: bool = true;
+    let mut expected_priv_key_ok = AtcaStatus::AtcaInvalidId;
+    let mut expected_priv_key_bad_1 = AtcaStatus::AtcaInvalidSize;
+    let mut expected_priv_key_bad_2 = AtcaStatus::AtcaBadParam;
+    let mut expected_pub_key_ok = AtcaStatus::AtcaSuccess;
+    let mut expected_pub_key_bad_1 = AtcaStatus::AtcaInvalidSize;
+    let mut expected_pub_key_bad_2 = AtcaStatus::AtcaInvalidId;
+    let mut expected_pub_key_bad_3 = AtcaStatus::AtcaBadParam;
+    let mut expected_aes_key_ok = AtcaStatus::AtcaBadParam;
+    let mut expected_aes_key_bad_1 = AtcaStatus::AtcaBadParam;
+
+    if !(device.is_configuration_locked() && device.is_data_zone_locked()) {
+        println!("\u{001b}[1m\u{001b}[33mConfiguration and/or Data zone not Locked!\u{001b}[0m ");
+        chip_is_locked = false;
+
+        expected_priv_key_ok = AtcaStatus::AtcaNotLocked;
+        expected_priv_key_bad_1 = AtcaStatus::AtcaNotLocked;
+        expected_priv_key_bad_2 = AtcaStatus::AtcaNotLocked;
+
+        expected_pub_key_ok = AtcaStatus::AtcaNotLocked;
+        expected_pub_key_bad_1 = AtcaStatus::AtcaNotLocked;
+        expected_pub_key_bad_2 = AtcaStatus::AtcaNotLocked;
+        expected_pub_key_bad_3 = AtcaStatus::AtcaNotLocked;
+
+        expected_aes_key_ok = AtcaStatus::AtcaNotLocked;
+        expected_aes_key_bad_1 = AtcaStatus::AtcaNotLocked;
+    }
+
+    if chip_is_locked && device.is_aes_enabled() {
+        expected_aes_key_ok = AtcaStatus::AtcaSuccess;
+        expected_aes_key_bad_1 = AtcaStatus::AtcaInvalidSize;
+    }
+
     let write_key_set_success = device.add_access_key(ENCRYPTION_KEY_SLOT, &write_key);
-    assert_eq!(write_key_set_success.to_string(), "AtcaSuccess");
-    if AtcaStatus::AtcaSuccess == write_key_set_success {
-        let priv_key_ok = device.import_key(KeyType::P256EccKey, &priv_key, 0x02);
-        let priv_key_bad_1 = device.import_key(KeyType::P256EccKey, &priv_key_bad, 0x00);
-        let priv_key_bad_2 = device.import_key(KeyType::P256EccKey, &priv_key, 0x01);
-
-        let pub_key_ok = device.import_key(KeyType::P256EccKey, &pub_key, 0x0B);
-        let pub_key_bad_1 = device.import_key(KeyType::P256EccKey, &pub_key_bad, 0x0B);
-        // slot number too low
-        let pub_key_bad_2 = device.import_key(KeyType::P256EccKey, &pub_key, 0x03);
-        // writing to a slot with a key type other than P256
-        let pub_key_bad_3 = device.import_key(KeyType::P256EccKey, &pub_key, 0x0C);
-
-        let mut expected_priv_key_ok = AtcaStatus::AtcaSuccess;
-        let mut expected_priv_key_bad_1 = AtcaStatus::AtcaInvalidSize;
-        let mut expected_priv_key_bad_2 = AtcaStatus::AtcaBadParam;
-        let mut expected_pub_key_ok = AtcaStatus::AtcaSuccess;
-        let mut expected_pub_key_bad_1 = AtcaStatus::AtcaInvalidSize;
-        let mut expected_pub_key_bad_2 = AtcaStatus::AtcaInvalidId;
-        let mut expected_pub_key_bad_3 = AtcaStatus::AtcaBadParam;
-        if !(device.is_configuration_locked() && device.is_data_zone_locked()) {
-            print!("\u{001b}[1m\u{001b}[33mConfiguration not Locked!\u{001b}[0m ");
-            expected_priv_key_ok = AtcaStatus::AtcaNotLocked;
-            expected_priv_key_bad_1 = AtcaStatus::AtcaNotLocked;
-            expected_priv_key_bad_2 = AtcaStatus::AtcaNotLocked;
-
-            expected_pub_key_ok = AtcaStatus::AtcaNotLocked;
-            expected_pub_key_bad_1 = AtcaStatus::AtcaNotLocked;
-            expected_pub_key_bad_2 = AtcaStatus::AtcaNotLocked;
-            expected_pub_key_bad_3 = AtcaStatus::AtcaNotLocked;
-        }
-
-        assert_eq!(priv_key_ok, expected_priv_key_ok);
-        assert_eq!(priv_key_bad_1, expected_priv_key_bad_1);
-        assert_eq!(priv_key_bad_2, expected_priv_key_bad_2);
-
-        assert_eq!(pub_key_ok, expected_pub_key_ok);
-        assert_eq!(pub_key_bad_1, expected_pub_key_bad_1);
-        assert_eq!(pub_key_bad_2, expected_pub_key_bad_2);
-        assert_eq!(pub_key_bad_3, expected_pub_key_bad_3);
+    if chip_is_locked && (AtcaStatus::AtcaSuccess == write_key_set_success) {
+        expected_priv_key_ok = AtcaStatus::AtcaSuccess;
     }
 
-    // TODO AES keys check
-    match is_chip_version_608(&device) {
-        Ok(true) => {
-            let aes_key_ok = device.import_key(KeyType::Aes, &aes_key, 0x09);
-            let aes_key_bad_1 = device.import_key(KeyType::Aes, &aes_key_bad, 0x09);
+    let priv_key_ok = device.import_key(KeyType::P256EccKey, &priv_key, 0x02);
+    let priv_key_bad_1 = device.import_key(KeyType::P256EccKey, &priv_key_bad, 0x00);
+    let priv_key_bad_2 = device.import_key(KeyType::P256EccKey, &priv_key, 0x01);
 
-            assert_eq!(aes_key_ok.to_string(), "AtcaSuccess");
-            assert_eq!(aes_key_bad_1.to_string(), "AtcaInvalidSize");
-        }
-        Ok(false) => (),
-        Err(err) => panic!("is_chip_version_608() error: {}", err.to_string()),
-    }
+    let pub_key_ok = device.import_key(KeyType::P256EccKey, &pub_key, 0x0B);
+    let pub_key_bad_1 = device.import_key(KeyType::P256EccKey, &pub_key_bad, 0x0B);
+    // slot number too low
+    let pub_key_bad_2 = device.import_key(KeyType::P256EccKey, &pub_key, 0x03);
+    // writing to a slot with a key type other than P256
+    let pub_key_bad_3 = device.import_key(KeyType::P256EccKey, &pub_key, 0x0C);
+
+    let aes_key_ok = device.import_key(KeyType::Aes, &aes_key, 0x09);
+    let aes_key_bad_1 = device.import_key(KeyType::Aes, &aes_key_bad, 0x09);
 
     assert_eq!(device.release().to_string(), "AtcaSuccess");
+
+    assert_eq!(write_key_set_success, AtcaStatus::AtcaSuccess);
+
+    assert_eq!(priv_key_ok, expected_priv_key_ok);
+    assert_eq!(priv_key_bad_1, expected_priv_key_bad_1);
+    assert_eq!(priv_key_bad_2, expected_priv_key_bad_2);
+
+    assert_eq!(pub_key_ok, expected_pub_key_ok);
+    assert_eq!(pub_key_bad_1, expected_pub_key_bad_1);
+    assert_eq!(pub_key_bad_2, expected_pub_key_bad_2);
+    assert_eq!(pub_key_bad_3, expected_pub_key_bad_3);
+
+    assert_eq!(aes_key_ok, expected_aes_key_ok);
+    assert_eq!(aes_key_bad_1, expected_aes_key_bad_1);
 }
 
 #[test]
@@ -385,34 +303,41 @@ fn get_pubkey() {
     ]
     .to_vec();
 
+    let mut chip_is_locked: bool = true;
     let get_key_ok_1 = device.get_public_key(0x00, &mut public_key);
     let sum: u16 = public_key.iter().fold(0, |s, &x| s + x as u16);
-    let mut get_key_ok_2 = AtcaStatus::AtcaUnknown;
     let get_key_bad_1 = device.get_public_key(0x01, &mut public_key);
 
     let mut expected_get_key_ok_1 = AtcaStatus::AtcaSuccess;
     let mut expected_get_key_ok_2 = AtcaStatus::AtcaSuccess;
     let mut expected_get_key_bad_1 = AtcaStatus::AtcaBadParam;
+    let mut expected_import_key = AtcaStatus::AtcaSuccess;
+
+    if !(device.is_configuration_locked() && device.is_data_zone_locked()) {
+        println!("\u{001b}[1m\u{001b}[33mConfiguration and/or Data zone not Locked!\u{001b}[0m ");
+        chip_is_locked = false;
 
-    if !device.is_configuration_locked() {
-        print!("\u{001b}[1m\u{001b}[33mConfiguration not Locked!\u{001b}[0m ");
         expected_get_key_ok_1 = AtcaStatus::AtcaNotLocked;
         expected_get_key_ok_2 = AtcaStatus::AtcaNotLocked;
         expected_get_key_bad_1 = AtcaStatus::AtcaNotLocked;
-    } else {
-        assert_eq!(public_key.len(), ATCA_ATECC_PUB_KEY_SIZE);
-        assert_ne!(sum, 0);
+        expected_import_key = AtcaStatus::AtcaNotLocked;
     }
 
-    if device.import_key(KeyType::P256EccKey, &public_key_write, 0x0B) == AtcaStatus::AtcaSuccess {
-        get_key_ok_2 = device.get_public_key(0x0B, &mut public_key);
-    }
+    let result_import_key = device.import_key(KeyType::P256EccKey, &public_key_write, 0x0B);
+    let get_key_ok_2 = device.get_public_key(0x0B, &mut public_key);
 
     assert_eq!(device.release().to_string(), "AtcaSuccess");
 
+    assert_eq!(result_import_key, expected_import_key);
+    if chip_is_locked {
+        assert_eq!(public_key.len(), ATCA_ATECC_PUB_KEY_SIZE);
+        assert_ne!(sum, 0);
+    }
+    if AtcaStatus::AtcaSuccess == get_key_ok_2 {
+        assert_eq!(public_key_write, public_key);
+    }
     assert_eq!(get_key_ok_1, expected_get_key_ok_1);
     assert_eq!(get_key_ok_2, expected_get_key_ok_2);
-    assert_eq!(public_key_write, public_key);
     assert_eq!(get_key_bad_1, expected_get_key_bad_1);
 }
 
@@ -449,7 +374,7 @@ fn export_key_aes() {
         expected_export_key_ok_1 = AtcaStatus::AtcaBadParam;
     }
     if !(device.is_configuration_locked() && device.is_data_zone_locked()) {
-        print!("\u{001b}[1m\u{001b}[33mConfiguration not Locked!\u{001b}[0m ");
+        println!("\u{001b}[1m\u{001b}[33mConfiguration and/or Data zone not Locked!\u{001b}[0m ");
         expected_export_key_bad_1 = AtcaStatus::AtcaNotLocked;
         expected_export_key_bad_2 = AtcaStatus::AtcaNotLocked;
         expected_import_key_result = AtcaStatus::AtcaNotLocked;
@@ -470,7 +395,7 @@ fn export_key_aes() {
     assert_eq!(export_key_bad_1, expected_export_key_bad_1);
     assert_eq!(export_key_bad_2, expected_export_key_bad_2);
 
-    assert_eq!(device_set_write_key.to_string(), "AtcaSuccess");
+    assert_eq!(device_set_write_key, AtcaStatus::AtcaSuccess);
     assert_eq!(import_key_result, expected_import_key_result);
     assert_eq!(export_key_ok_1, expected_export_key_ok_1);
     if (AtcaStatus::AtcaNotLocked != expected_export_key_ok_1) && device.is_aes_enabled() {
@@ -483,6 +408,7 @@ fn export_key_aes() {
 fn sign_verify_hash() {
     let device = test_setup();
 
+    let mut chip_is_fully_locked: bool = true;
     let hash: Vec<u8> = vec![0xA5; 32];
     let internal_sig = SignEcdsaParam {
         is_invalidate: false,
@@ -518,21 +444,26 @@ fn sign_verify_hash() {
     let mut expected_get_pub_key_result = AtcaStatus::AtcaSuccess;
     let mut expected_verify_internal_result = AtcaStatus::AtcaSuccess;
     if !device.is_configuration_locked() {
+        println!("\u{001b}[1m\u{001b}[33mConfiguration not Locked!\u{001b}[0m ");
         expected_get_pub_key_result = AtcaStatus::AtcaNotLocked;
     }
     if !(device.is_configuration_locked() && device.is_data_zone_locked()) {
-        print!("\u{001b}[1m\u{001b}[33mConfiguration not Locked!\u{001b}[0m ");
+        println!("\u{001b}[1m\u{001b}[33mData zone not Locked!\u{001b}[0m ");
+        chip_is_fully_locked = false;
+
         expected_sign_internal = AtcaStatus::AtcaNotLocked;
         expected_verify_external_result = AtcaStatus::AtcaNotLocked;
         expected_sign_external = AtcaStatus::AtcaNotLocked;
         expected_verify_internal_result = AtcaStatus::AtcaNotLocked;
-    } else {
-        assert_eq!(signature.len(), ATCA_SIG_SIZE);
-        assert_eq!(is_verified, true);
+        expected_get_pub_key_result = AtcaStatus::AtcaNotLocked;
     }
 
     assert_eq!(device.release().to_string(), "AtcaSuccess");
 
+    if chip_is_fully_locked {
+        assert_eq!(signature.len(), ATCA_SIG_SIZE);
+        assert_eq!(is_verified, true);
+    }
     assert_eq!(sign_internal, expected_sign_internal);
     assert_eq!(verify_external_result, expected_verify_external_result);
 
@@ -541,6 +472,49 @@ fn sign_verify_hash() {
     assert_eq!(verify_internal_result, expected_verify_internal_result);
 }
 
+#[test]
+#[serial]
+fn gen_key_sign_hash() {
+    const ENCRYPTION_KEY_SLOT: u8 = 0x06;
+
+    let device = test_setup();
+
+    let write_key = [
+        0x4D, 0x50, 0x72, 0x6F, 0x20, 0x49, 0x4F, 0x20, 0x4B, 0x65, 0x79, 0x20, 0x9E, 0x31, 0xBD,
+        0x05, 0x82, 0x58, 0x76, 0xCE, 0x37, 0x90, 0xEA, 0x77, 0x42, 0x32, 0xBB, 0x51, 0x81, 0x49,
+        0x66, 0x45,
+    ];
+
+    let mut expected_device_sha = AtcaStatus::AtcaSuccess;
+    let mut expected_device_gen_key = AtcaStatus::AtcaSuccess;
+    let mut expected_device_sign_hash = AtcaStatus::AtcaSuccess;
+
+    if !device.is_configuration_locked() {
+        println!("\u{001b}[1m\u{001b}[33Configuration not Locked!\u{001b}[0m ");
+        expected_device_sha = AtcaStatus::AtcaNotLocked;
+        expected_device_gen_key = AtcaStatus::AtcaNotLocked;
+    }
+    if !device.is_data_zone_locked() {
+        println!("\u{001b}[1m\u{001b}[33mData zone not Locked!\u{001b}[0m ");
+        expected_device_sign_hash = AtcaStatus::AtcaNotLocked;
+    }
+
+    let device_set_write_key = device.add_access_key(ENCRYPTION_KEY_SLOT, &write_key);
+
+    let mut digest: Vec<u8> = Vec::new();
+    let device_sha = device.sha("Bob wrote this message.".as_bytes().to_vec(), &mut digest);
+    let device_gen_key = device.gen_key(KeyType::P256EccKey, 0);
+    let mut signature = vec![0u8; ATCA_SIG_SIZE];
+    let device_sign_hash = device.sign_hash(SignMode::External(digest), 0, &mut signature);
+
+    assert_eq!(device.release().to_string(), "AtcaSuccess");
+
+    assert_eq!(device_set_write_key, AtcaStatus::AtcaSuccess);
+    assert_eq!(device_sha, expected_device_sha);
+    assert_eq!(device_gen_key, expected_device_gen_key);
+    assert_eq!(device_sign_hash, expected_device_sign_hash);
+}
+
 #[test]
 #[serial]
 fn cmp_config_zone() {
@@ -560,8 +534,8 @@ fn cmp_config_zone() {
 
     assert_eq!(device.release().to_string(), "AtcaSuccess");
 
-    assert_eq!(device_read_config_zone.to_string(), "AtcaSuccess");
-    assert_eq!(device_cmp_config_zone.to_string(), "AtcaSuccess");
+    assert_eq!(device_read_config_zone, AtcaStatus::AtcaSuccess);
+    assert_eq!(device_cmp_config_zone, AtcaStatus::AtcaSuccess);
     assert_eq!(same_config, true);
 }
 
@@ -569,6 +543,7 @@ fn cmp_config_zone() {
 #[serial]
 fn is_configuration_locked() {
     let device = test_setup();
+
     let is_locked = device.is_configuration_locked();
 
     assert_eq!(device.release().to_string(), "AtcaSuccess");
@@ -580,6 +555,7 @@ fn is_configuration_locked() {
 #[serial]
 fn is_data_zone_locked() {
     let device = test_setup();
+
     let is_locked = device.is_data_zone_locked();
 
     assert_eq!(device.release().to_string(), "AtcaSuccess");
@@ -590,8 +566,9 @@ fn is_data_zone_locked() {
 #[test]
 #[serial]
 fn get_config_from_config_zone() {
-    let mut config_data = Vec::new();
     let device = test_setup();
+
+    let mut config_data = Vec::new();
     let device_atcab_read_config_zone = device.read_config_zone(&mut config_data);
 
     config_data[88] = 0b10111111;
@@ -603,7 +580,7 @@ fn get_config_from_config_zone() {
 
     assert_eq!(device.release().to_string(), "AtcaSuccess");
 
-    assert_eq!(device_atcab_read_config_zone.to_string(), "AtcaSuccess");
+    assert_eq!(device_atcab_read_config_zone, AtcaStatus::AtcaSuccess);
     assert_eq!(slots.len(), usize::from(ATCA_ATECC_SLOTS_COUNT));
     assert_eq!(slots[0].id, 0);
     assert_eq!(slots[15].id, 15);
@@ -618,12 +595,13 @@ fn get_config_from_config_zone() {
 #[serial]
 fn get_config() {
     let device = test_setup();
+
     let mut slots: Vec<AtcaSlot> = Vec::new();
     let get_config = device.get_config(&mut slots);
 
     assert_eq!(device.release().to_string(), "AtcaSuccess");
 
-    assert_eq!(get_config.to_string(), "AtcaSuccess");
+    assert_eq!(get_config, AtcaStatus::AtcaSuccess);
     assert_eq!(slots.len(), ATCA_ATECC_SLOTS_COUNT as usize);
 }
 
@@ -631,6 +609,7 @@ fn get_config() {
 #[serial]
 fn info_cmd() {
     let device = test_setup();
+
     let mut result_key_valid = AtcaStatus::AtcaSuccess;
     let mut result_revision = AtcaStatus::AtcaSuccess;
     let mut revision: Vec<u8> = Vec::new();
@@ -647,39 +626,9 @@ fn info_cmd() {
 
     assert_eq!(device.release().to_string(), "AtcaSuccess");
 
-    assert_eq!(result_key_valid.to_string(), "AtcaUnimplemented");
+    assert_eq!(result_key_valid, AtcaStatus::AtcaUnimplemented);
     assert_eq!(revision.len(), 4);
-    assert_eq!(result_revision.to_string(), "AtcaSuccess");
-}
-
-#[test]
-#[serial]
-fn gen_key_sign_hash() {
-    const ENCRYPTION_KEY_SLOT: u8 = 0x06;
-    let device = test_setup();
-
-    let write_key = [
-        0x4D, 0x50, 0x72, 0x6F, 0x20, 0x49, 0x4F, 0x20, 0x4B, 0x65, 0x79, 0x20, 0x9E, 0x31, 0xBD,
-        0x05, 0x82, 0x58, 0x76, 0xCE, 0x37, 0x90, 0xEA, 0x77, 0x42, 0x32, 0xBB, 0x51, 0x81, 0x49,
-        0x66, 0x45,
-    ];
-
-    let device_set_write_key = device.add_access_key(ENCRYPTION_KEY_SLOT, &write_key);
-
-    let mut digest: Vec<u8> = Vec::new();
-    let device_sha = device.sha("Bob wrote this message.".as_bytes().to_vec(), &mut digest);
-
-    let device_gen_key = device.gen_key(KeyType::P256EccKey, 0);
-    let mut signature = vec![0u8; ATCA_SIG_SIZE];
-
-    let device_sign_hash = device.sign_hash(SignMode::External(digest), 0, &mut signature);
-
-    assert_eq!(device.release().to_string(), "AtcaSuccess");
-
-    assert_eq!(device_set_write_key.to_string(), "AtcaSuccess");
-    assert_eq!(device_sha.to_string(), "AtcaSuccess");
-    assert_eq!(device_gen_key.to_string(), "AtcaSuccess");
-    assert_eq!(device_sign_hash.to_string(), "AtcaSuccess");
+    assert_eq!(result_revision, AtcaStatus::AtcaSuccess);
 }
 
 #[test]
@@ -709,15 +658,15 @@ fn add_get_and_flush_access_keys() {
 
     let device_add_key_ok_1 = device.add_access_key(OK_KEY_IDX_1, &test_key_1);
     let mut device_get_key_ok_1 = vec![0; ATCA_KEY_SIZE];
-    let mut _result = device.get_access_key(OK_KEY_IDX_1, &mut device_get_key_ok_1);
+    let result_get_key_ok_1 = device.get_access_key(OK_KEY_IDX_1, &mut device_get_key_ok_1);
 
     let device_add_key_ok_2 = device.add_access_key(OK_KEY_IDX_1, &test_key_2);
     let mut device_get_key_ok_2 = vec![0; ATCA_KEY_SIZE];
-    _result = device.get_access_key(OK_KEY_IDX_1, &mut device_get_key_ok_2);
+    let result_get_key_ok_2 = device.get_access_key(OK_KEY_IDX_1, &mut device_get_key_ok_2);
 
     let device_add_key_ok_3 = device.add_access_key(OK_KEY_IDX_2, &test_key_1);
     let mut device_get_key_ok_3 = vec![0; ATCA_KEY_SIZE];
-    _result = device.get_access_key(OK_KEY_IDX_2, &mut device_get_key_ok_3);
+    let result_get_key_ok_3 = device.get_access_key(OK_KEY_IDX_2, &mut device_get_key_ok_3);
 
     let device_add_key_bad_1 = device.add_access_key(BAD_KEY_IDX_3, &test_key_1);
     let device_add_key_bad_2 = device.add_access_key(OK_KEY_IDX_1, &test_key_1[0..=25]);
@@ -731,16 +680,19 @@ fn add_get_and_flush_access_keys() {
 
     assert_eq!(device.release().to_string(), "AtcaSuccess");
 
-    assert_eq!(device_add_key_ok_1.to_string(), "AtcaSuccess");
+    assert_eq!(result_get_key_ok_1, AtcaStatus::AtcaSuccess);
+    assert_eq!(result_get_key_ok_2, AtcaStatus::AtcaSuccess);
+    assert_eq!(result_get_key_ok_3, AtcaStatus::AtcaSuccess);
+    assert_eq!(device_add_key_ok_1, AtcaStatus::AtcaSuccess);
     assert_eq!(device_get_key_ok_1, test_key_1);
-    assert_eq!(device_add_key_ok_2.to_string(), "AtcaSuccess");
+    assert_eq!(device_add_key_ok_2, AtcaStatus::AtcaSuccess);
     assert_eq!(device_get_key_ok_2, test_key_2);
-    assert_eq!(device_add_key_ok_3.to_string(), "AtcaSuccess");
+    assert_eq!(device_add_key_ok_3, AtcaStatus::AtcaSuccess);
     assert_eq!(device_get_key_ok_3, test_key_1);
 
-    assert_eq!(device_add_key_bad_1.to_string(), "AtcaInvalidId");
-    assert_eq!(device_add_key_bad_2.to_string(), "AtcaInvalidSize");
-    assert_eq!(device_get_key_bad_1.to_string(), "AtcaInvalidId");
-    assert_eq!(device_get_key_bad_2.to_string(), "AtcaInvalidId");
-    assert_eq!(device_get_key_bad_3.to_string(), "AtcaInvalidId");
+    assert_eq!(device_add_key_bad_1, AtcaStatus::AtcaInvalidId);
+    assert_eq!(device_add_key_bad_2, AtcaStatus::AtcaInvalidSize);
+    assert_eq!(device_get_key_bad_1, AtcaStatus::AtcaInvalidId);
+    assert_eq!(device_get_key_bad_2, AtcaStatus::AtcaInvalidId);
+    assert_eq!(device_get_key_bad_3, AtcaStatus::AtcaInvalidId);
 }
