@@ -3,14 +3,15 @@ use serial_test::serial;
 
 // Types
 use super::{
-    AtcaDeviceType, AtcaIface, AtcaIfaceCfg, AtcaIfaceI2c, AtcaSlot, AtcaStatus, AteccDevice,
-    InfoCmdType, KeyType, NonceTarget, SignEcdsaParam, SignMode, VerifyEcdsaParam, VerifyMode,
+    AeadAlgorithm, AeadParam, AtcaDeviceType, AtcaIface, AtcaIfaceCfg, AtcaIfaceI2c, AtcaSlot,
+    AtcaStatus, AteccDevice, InfoCmdType, KeyType, NonceTarget, SignEcdsaParam, SignMode,
+    VerifyEcdsaParam, VerifyMode,
 };
 // Constants
 use super::{
-    ATCA_AES_KEY_SIZE, ATCA_ATECC_CONFIG_BUFFER_SIZE, ATCA_ATECC_PUB_KEY_SIZE,
-    ATCA_ATECC_SLOTS_COUNT, ATCA_NONCE_NUMIN_SIZE, ATCA_RANDOM_BUFFER_SIZE, ATCA_SIG_SIZE,
-    ATCA_ZONE_CONFIG,
+    ATCA_AES_DATA_SIZE, ATCA_AES_GCM_IV_STD_LENGTH, ATCA_AES_KEY_SIZE,
+    ATCA_ATECC_CONFIG_BUFFER_SIZE, ATCA_ATECC_PUB_KEY_SIZE, ATCA_ATECC_SLOTS_COUNT,
+    ATCA_NONCE_NUMIN_SIZE, ATCA_RANDOM_BUFFER_SIZE, ATCA_SIG_SIZE, ATCA_ZONE_CONFIG,
 };
 // Functions
 use super::setup_atecc_device;
@@ -19,6 +20,13 @@ use super::hw_impl;
 
 #[cfg(not(feature = "software-backend"))]
 mod hw_backend;
+#[cfg(not(feature = "software-backend"))]
+mod hw_backend_aes_ccm;
+#[cfg(not(feature = "software-backend"))]
+mod hw_backend_aes_gcm;
+#[cfg(not(feature = "software-backend"))]
+mod hw_backend_common;
+
 #[cfg(feature = "software-backend")]
 mod sw_backend;
 
@@ -59,7 +67,7 @@ fn random() {
     }
     #[cfg(not(feature = "software-backend"))]
     {
-        let device = hw_backend::test_setup();
+        let device = hw_backend_common::test_setup();
 
         let mut rand_out = Vec::new();
         let device_random = device.random(&mut rand_out);
@@ -82,7 +90,7 @@ fn read_config_zone() {
     #[cfg(feature = "software-backend")]
     let device = sw_backend::test_setup("always-success".to_owned());
     #[cfg(not(feature = "software-backend"))]
-    let device = hw_backend::test_setup();
+    let device = hw_backend_common::test_setup();
 
     let mut config_data = Vec::new();
     let device_read_config_zone = device.read_config_zone(&mut config_data);
